package src;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Random;
 
/**
 *
 * @author fco.dominguez
 * @author juan.alvarez
 */

public class Partida extends Observable {

    // <editor-fold defaultstate="collapsed" desc=" UML Marker "> 
    // #[regen=yes,id=DCE.AAD7E6C1-1036-7F57-308D-474E893A6D91]
    // </editor-fold> 
    private Tablero tablero;//tablero de la partida

    // <editor-fold defaultstate="collapsed" desc=" UML Marker "> 
    // #[regen=yes,id=DCE.AB81F2E2-1DDE-961D-29E9-10BE7ACDDD6B]
    // </editor-fold> 
    private ArrayList<Jugador> jugadores;//array que contiene los jugadores de la partida
    private ArrayList<CartaTerritorio> cartasTer;//array que contiene las cartas de Territorio
    private ArrayList<Territorio> capitales; //lista de capitales para el modo Capital
    private int turno = 0;//variable para saber que jugador tiene el turno
    private int ronda = 0; //variable que indica en que ronda va la partida
    private Territorio terrSeleccionado = null;
    private Territorio terrSeleccionadoDest = null;
    private boolean modoInicializar = false;
    private boolean modoUbicar = false;
    private boolean modoMoverTropas = false;
    private boolean modoAtaque = false;
    private boolean mostrarAtaque = false;
    private boolean mostrarMoverTropas = false;
    private boolean modoInvasion = false;
    private boolean cambiarCartas = false; //variable que indica que el usuario puede cambiar sus cartas
    private boolean recibeCarta = false; //variable que indica que el jugador recibira una carta en su turno
    private Jugador ganador; //variable que indicará quien es el ganador del Juego
    private int modoJuego; //variable que indica el modo de Juego de la partida
    private int rondaMax = 0;
    private boolean finJuego; //variable que indica cuando sea ha terminado la partida
    private boolean capitalesVisibles = true;

    public Partida() {
        jugadores = new ArrayList<Jugador>();
        cartasTer = new ArrayList<CartaTerritorio>();
    }

    //Añade un jugador a la lista
    public void anadirJugador(String nombre, int color) {
        jugadores.add(new Jugador(nombre, color));
    }

    //ordena los jugadores segun su tirada de dado
    public void ordenarJug() {
        Jugador auxiliar;

        for (int i = 0; i < jugadores.size(); i++) {
            for (int j = i; j < jugadores.size(); j++) {
                if (jugadores.get(i).getDadoComienzo().getValor() < jugadores.get(j).getDadoComienzo().getValor()) {
                    auxiliar = jugadores.get(j);
                    jugadores.set(j, jugadores.get(i));
                    jugadores.set(i, auxiliar);
                }
            }
        }
    }

    public void quitarJugador(Jugador jug) {
        jugadores.remove(jug);
    }
    //Metodo que trata el territorio seleccionado por label
    public void seleccionarTerritorio(int i) {
        //si no se ha acabado el juego
        if (!isFinJuego()) {
            //Comprobamos si estamos en modo inicializar
            if (modoInicializar) {
                System.out.println("Modo inicializar");
                //Seleccionamos el territorio pulsado
                terrSeleccionado = getTablero().getTerritorio().get(i);
                //si no ha habido problemas al ubicar pasamos el turno al siguiente jugador
                if (jugadores.get(turno).ubicarTropas(getTerrSeleccionado())) {
                    pasarTurno();
                    //si el jugador nuevo no tiene tropas en reserva es que ya se han ubicado todas
                    if (jugadores.get(turno).getTropasReserva() == 0) {
                        //quitamos el modo inicializar
                        setModoInicializar(false);
                        //Pasamos de ronda 0 a 1 y damos turno al ultimo jugador, para que en nueva ronda se lo pase al 0
                        setTurno(jugadores.size() - 1);
                        nuevaRonda();
                    }
                }
            } //si estamos en modo ubicar
            else if (modoUbicar) {
                System.out.println("Modo ubicar");
                //Seleccionamos el territorio pulsado
                terrSeleccionado = getTablero().getTerritorio().get(i);
                //ubicamos las tropas de 1 en 1 hasta que no queden mas
                if (jugadores.get(turno).ubicarTropas(getTerrSeleccionado())) {
                    //si ha ubicado una tropa, ya no puede cambiar las cartas en esta ronda
                    setCambiarCartas(false);
                    terrSeleccionado = null;
                    //si ya ha ubicado todas las tropas, ya puede atacar
                    if (jugadores.get(turno).getTropasReserva() == 0) {
                        setModoUbicar(false);
                        //pasamos al modo ataque
                        setModoAtaque(true);
                    }
                }
            } //si estamos en modo ataque
            else if (modoAtaque) {
                System.out.println("Modo ataque");
                //si es el territorio origen de la accion, lo marcamos
                if ((terrSeleccionado == null) || (terrSeleccionado.getPropietarioTer() != jugadores.get(turno))) {
                    terrSeleccionado = getTablero().getTerritorio().get(i);
                } //si ya hay un terrSeleccionado previamente, marcaremos el terrDestino
                else {
                    terrSeleccionadoDest = getTablero().getTerritorio().get(i);
                    //el origen debe ser del jugador actual
                    if (terrSeleccionado.getPropietarioTer() == jugadores.get(turno)) {
                        //Comprobamos si puede atacar con los territorios seleccionados
                        if (getJugador().get(turno).puedeAtacar(terrSeleccionado, terrSeleccionadoDest)) {
                            //marcamos que se muestre la ventana de ataque
                            setMostrarAtaque(true);
                        } else {
                            terrSeleccionado = terrSeleccionadoDest;
                            terrSeleccionadoDest = null;
                        }
                    }
                }
            } //si estamos en modo movertropas
            else if (modoMoverTropas) {
                System.out.println("Modo movertropas");
                //si es el territorio origen de la accion, lo marcamos
                if (terrSeleccionado == null) {
                    terrSeleccionado = getTablero().getTerritorio().get(i);
                } //si ya hay un terrSeleccionado previamente, marcaremos el terrDestino
                else {
                    terrSeleccionadoDest = getTablero().getTerritorio().get(i);
                    //el origen debe ser del jugador actual y tener un numero minimo de 2 tropas, y no ser el mismo territorio
                    if (jugadores.get(turno).puedeMover(terrSeleccionado, terrSeleccionadoDest)) {
                        //marcamos que se muestre la ventana de mover tropas
                        setMostrarMoverTropas(true);
                    } else {
                        terrSeleccionado = terrSeleccionadoDest;
                        terrSeleccionadoDest = null;
                    }

                }
            }
        }

        //avisamos a los observadores de los cambios
        setChanged();
        notifyObservers();
    }

    public void puedeMover() {
    }

    public void moverTropas(Territorio origen, Territorio destino, int numTropas) {
        if (jugadores.get(turno).moverTropas(origen, destino, numTropas)) {
            //si no hay problemas al mover las tropas pasamos el turno al siguiente jugador
            //ya que el jugador solo puede mover una vez y es lo ultimo que puede hacer
            if (turno + 1 == jugadores.size()) {
                nuevaRonda();
            } else {
                pasarTurno();
            }
        } //si ha habido fallos inicializamos los territorios
        else {
            terrSeleccionado = null;
            terrSeleccionadoDest = null;
        }
        
        //avisamos a los observadores de los cambios
        setChanged();
        notifyObservers();
    }

    //metodo que mueve tropas de un territorio a otro que acaba de ser invadido
    public void invadirTer(Territorio origen, Territorio destino, int numTropas) {
        origen.setNumEjercito(origen.getNumEjercito() - numTropas);
        destino.setNumEjercito(destino.getNumEjercito() + numTropas);

        terrSeleccionado = null;
        terrSeleccionadoDest = null;

        //avisamos a los observadores de los cambios
        setChanged();
        notifyObservers();
    }

    public CartaTerritorio cogerCarta() {
        Random rnd = new Random();
        //cogemos un valor aleatorio segun el tamaño de la baraja
        int numCarta = rnd.nextInt(cartasTer.size());

        //cogemos la carta
        CartaTerritorio cogida = cartasTer.get(numCarta);

        //y la quitamos de la baraja
        cartasTer.remove(numCarta);

        //devolvemos la carta cogida
        return cogida;
    }

    public void atacar(Territorio origen, Territorio destino, int numAtq, int numDef) {
        //realizamos el ataque
        ArrayList<Dado> dadAtq = new ArrayList<Dado>(numAtq);
        ArrayList<Dado> dadDef = new ArrayList<Dado>(numDef);
        for (int i = 0; i < numAtq; i++) {
            dadAtq.add(new Dado('a'));
        }

        for (int i = 0; i < numDef; i++) {
            dadDef.add(new Dado('d'));
        }

        origen.getPropietarioTer().setDadoAta(dadAtq);
        origen.getPropietarioTer().setDadoDef(dadDef);
        destino.getPropietarioTer().setDadoDef(dadDef);
        destino.getPropietarioTer().setDadoAta(dadAtq);

        //jugador auxiliar
        Jugador perdedor = terrSeleccionadoDest.getPropietarioTer();

        jugadores.get(turno).compararDados(origen, destino);


        //si el jugador ha ganado el territorio se le dara una carta
        if (terrSeleccionado.getPropietarioTer() == terrSeleccionadoDest.getPropietarioTer()) {
            //permitimos al jugador mover las tropas hacia el territorio invadido
            setModoInvasion(true);
            //si puede recibir carta, se le da, solo 1 por ronda para ese jugador
            //y no tiene ya 6 cartas
            if ((recibeCarta) && (jugadores.get(turno).getCartaTerritorio().size() < 7)) {
                //cogemos una carta de la baraja
                CartaTerritorio carta = cogerCarta();
                //se la damos al jugador
                jugadores.get(turno).anadirCartaTer(carta);
                //solo recibe una carta por turno
                recibeCarta = false;
            }
            //si es el modo capital
            if (getModoJuego() == 1) {
                //si el territorio invadido es una capital
                if (terrSeleccionadoDest.isEsCapital()) {
                    //se la añadimos al jugador
                    jugadores.get(turno).anadirCapital(terrSeleccionadoDest);
                    //y se la quitamos al perdedor
                    perdedor.quitarCapital(terrSeleccionadoDest);
                }
            }
            comprobarContinentes();
            if (comprobarFin()) {
                System.out.println("Fin de juego!");
                setFinJuego(true);
            }

        }

        //avisamos a los observadores de los cambios
        setChanged();
        notifyObservers();

    }

    private void aleatorioJugador() {
    }

    // <editor-fold defaultstate="collapsed" desc=" UML Marker "> 
    // #[regen=yes,regenBody=yes,id=DCE.4F6F0C3E-AB37-B9A9-E685-CABB2D1F3D76]
    // </editor-fold> 
    public ArrayList<Jugador> getJugador() {
        return jugadores;
    }

// <editor-fold defaultstate="collapsed" desc=" UML Marker "> 
// #[regen=yes,regenBody=yes,id=DCE.F568669E-193B-52C1-E637-0CD07A354CC6]
// </editor-fold> 
    public void setJugador(ArrayList<Jugador> val) {
        jugadores = val;
    }

// <editor-fold defaultstate="collapsed" desc=" UML Marker "> 
// #[regen=yes,regenBody=yes,id=DCE.E36CBC13-E609-242B-94E7-4E12D110E79B]
// </editor-fold> 
    public Tablero getTablero() {
        return tablero;
    }

// <editor-fold defaultstate="collapsed" desc=" UML Marker "> 
// #[regen=yes,regenBody=yes,id=DCE.0D1D7706-642D-57EF-2648-DF15DC30B9CC]
// </editor-fold> 
    public void setTablero(Tablero val) {
        tablero = val;
    }

//metodo que crea la baraja con las cartas
    public void inicializarCartas() {
        ArrayList<Territorio> territorios = getTablero().getTerritorio();
        cartasTer = new ArrayList<CartaTerritorio>(44);

        cartasTer.add(new CartaTerritorio(territorios.get(23), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(4), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(18), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(12), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(2), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(38), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(41), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(31), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(36), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(33), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(14), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(11), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(15), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(20), 10));
        cartasTer.add(new CartaTerritorio(territorios.get(24), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(19), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(9), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(7), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(35), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(27), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(8), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(0), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(32), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(5), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(13), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(25), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(21), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(17), 5));
        cartasTer.add(new CartaTerritorio(territorios.get(30), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(10), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(22), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(26), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(3), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(1), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(39), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(16), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(34), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(6), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(28), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(37), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(40), 1));
        cartasTer.add(new CartaTerritorio(territorios.get(29), 1));

        //creamos los 2 comodines y los añadimos al a baraja
        cartasTer.add(new CartaTerritorio("Comodin"));
        cartasTer.add(new CartaTerritorio("Comodin"));
    }
//metodo que comprueba si un jugador tiene un continente entero
    public void comprobarContinentes() {
        //para cada continente
        for (int i = 0; i < getTablero().getContinente().size(); i++) {
            boolean esDueno = true;
            int j = 0;
            Jugador dueno = getTablero().getContinente().get(i).getTerritorio().get(j).getPropietarioTer();
            j++;
//mientras se recorran todos los paises y el jugador sea el dueño de cada uno
            while ((j < getTablero().getContinente().get(i).getTerritorio().size()) && (esDueno)) {
                //comprobamos si es dueño del territorio siguiente, si no lo es devolvemos falso
                if (getTablero().getContinente().get(i).getTerritorio().get(j).getPropietarioTer() != dueno) {
                    esDueno = false;
                    //ponemos como dueño del continente a null
                    getTablero().getContinente().get(i).setPropietarioCont(null);
                }
                j++;
            }
            if (esDueno) {
                //si tiene todos los paises lo marcamos como dueño
                getTablero().getContinente().get(i).setPropietarioCont(dueno);
                //añadimos el continente a su lista si no estaba previamente
                if (!dueno.getContinente().contains(getTablero().getContinente().get(i))) {
                    dueno.anadirCont(getTablero().getContinente().get(i));
                }
            } else {
                //si no lo es, lo ponemos a nulo y lo quitamos si estuviera
                if (dueno.getContinente().contains(getTablero().getContinente().get(i))) {
                    dueno.quitarCont(getTablero().getContinente().get(i));
                }
                getTablero().getContinente().get(i).setPropietarioCont(null);
            }
        }
    }
    //Empieza el juego!
    public void jugar() throws IOException {
        //creamos las cartas de Territorio
        inicializarCartas();

        //inicializamos las tropas minimas
        inicializarTropas();

        prepararModo();

        //ademas comprobamos los continentes
        comprobarContinentes();

//marcamos la ronda 0, que indica que los jugadores aun estan ubicando sus tropas iniciales
        setRonda(0);

        //le damos el turno al jugador 1
        setTurno(0);

        //Activamos el modo inicializar
        setModoInicializar(true);
    }

    public void cambiarCartas(CartaTerritorio car1, CartaTerritorio car2, CartaTerritorio car3) {
        //si el cambio es correcto, ya no puede cambiar cartas este turno
        if (jugadores.get(turno).cambiarCartas(car1, car2, car3)) {
            cambiarCartas = false;
        }
        //avisamos a los observadores de los cambios
        setChanged();
        notifyObservers();
    }

    public void rearmamiento(int jugador) {
        //Este metodo comprobará los territorios que posee un jugador y añadira
        //el numero de tropas de reserva que le correspondan.

        //ademas comprobamos los continentes
        comprobarContinentes();

        int i = jugador;
        //Variable auxiliar para almacenar las tropas
        int tropasNuevas = jugadores.get(i).getTropasReserva();
        //Cogemos el numero de territorios que tiene y calculamos las tropas que
        //le corresponden, ignorando el resto de la division por 3
        tropasNuevas += (jugadores.get(i).getTerritorio().size()) / 3;

        //Ahora comprobamos los continentes que posee y le añadimos
        //las tropas segun le correspondan.
        if (jugadores.get(i).getContinente() != null) {
            for (int j = 0; j < jugadores.get(i).getContinente().size(); j++) {
                tropasNuevas += jugadores.get(i).getContinente().get(j).getTropasPremio();
                System.out.println("Continente: " + jugadores.get(i).getContinente().get(j).getTropasPremio());
            }

        }

        // Cada jugador recibe al minimo 3 tropas por turno, aunque no tenga
        // territorios suficientes.
        if (tropasNuevas < 3) {
            tropasNuevas = 3;
        }

        //Añadimos las tropas a la reserva del jugador
        jugadores.get(i).setTropasReserva(tropasNuevas);
    }

    public void inicializarTropas() throws IOException {
        //Este metodo ubicara el número de tropas iniciales en el primer turno
        //de juego.
        //ahora mismo este metodo realiza la asiganacion de los territorios automaticamente
        //es una de la posibilidades que le daremos al jugador


        int numJugadores = jugadores.size();
        //calculo de cuantos ejercitos se le debe asignar a cada jugador al comienzo del juego.
        int numeroEjercitos = 50 - (numJugadores * 5);
        //asignamos a cada jugador el numero de ejercitos correspondientes.
        for (int i = 0; i < numJugadores; i++) {
            jugadores.get(i).setTropasReserva(numeroEjercitos);
        }
//empezamos a repartir los territorios de forma aleatoria entre los jugadores.
        int n = 42;  //numeros aleatorios

        int k = n;  //auxiliar;

        int[] numeros = new int[n];
        int[] resultado = new int[n];
        Random rnd = new Random();
        int res;


        //se rellena una matriz ordenada del 1 al 42(1..n)
        for (int i = 0; i < n; i++) {
            numeros[i] = i;
        }

        int j = 0;
        ArrayList<Territorio> repTerritorio = tablero.getTerritorio();
        int h;
        for (int i = 0; i < n; i++) {
            res = rnd.nextInt(k);
            resultado[i] = numeros[res];
            h = numeros[res];
            numeros[res] = numeros[k - 1];
            k--;

            jugadores.get(j).setTerritorio(repTerritorio.get(h));//asignamos el territorio al jugador

            repTerritorio.get(h).setPropietarioTer(jugadores.get(j));//asignamos el propietario al territorio

            jugadores.get(j).setTropasReserva((jugadores.get(j).getTropasReserva()) - 1); //Le quitamos la tropa para asignarla

            repTerritorio.get(h).setNumEjercito(1);//asignamos un ejercito al territorio

            j++;

            if (j >= numJugadores) {
                j = 0;
            }

        }
    }

    public int getTurno() {
        return turno;
    }

    public int getRonda() {
        return ronda;
    }

//metodo que se encarga de pasar el turno de un juegador a otro
    public void pasarTurno() {
        //si era el ultimo jugador, le damos el turno al jugador 1

        if (turno + 1 == jugadores.size()) {
            turno = 0;
        } else {
            turno++;
        }
        //ademas nos saltamos los jugadores neutrales
        while (jugadores.get(turno).isNeutral()) {
            if (turno + 1 == jugadores.size()) {
                turno = 0;
            } else {
                turno++;
            }
        }

//iniciamos los modos
        if (!modoInicializar) {
            rearmamiento(turno);

            modoUbicar = true;
            modoAtaque = false;
            modoMoverTropas = false;
            mostrarMoverTropas = false;
            mostrarAtaque = false;
            setCambiarCartas(true);
            recibeCarta = true;

            if (comprobarFin()) {
                System.out.println("Fin de juego!");
                setFinJuego(true);
            }
        }

        //inicializamos los territorios auxiliares
        terrSeleccionado = null;
        terrSeleccionadoDest = null;

        //avisamos a los observadores de los cambios
        setChanged();
        notifyObservers();
    }

    public void setTurno(int t) {
        this.turno = t;
    }

    public boolean isModoUbicar() {
        return modoUbicar;
    }

    public void setModoUbicar(boolean modoUbicar) {
        this.modoUbicar = modoUbicar;
    }

    public Territorio getTerrSeleccionado() {
        return terrSeleccionado;
    }

    public void setTerrSeleccionado(Territorio terrSeleccionado) {
        this.terrSeleccionado = terrSeleccionado;
    }

    public Territorio getTerrSeleccionadoDest() {
        return terrSeleccionadoDest;
    }

    public void setTerrSeleccionadoDest(Territorio terrSeleccionadoDest) {
        this.terrSeleccionadoDest = terrSeleccionadoDest;
    }

    public boolean isModoMoverTropas() {
        return modoMoverTropas;
    }

    public void setModoMoverTropas(boolean modoMoverTropas) {
        this.modoMoverTropas = modoMoverTropas;
    }

    public boolean isModoAtaque() {
        return modoAtaque;
    }

    public void setModoAtaque(boolean modoAtaque) {
        this.modoAtaque = modoAtaque;
    }

    public boolean isModoInicializar() {
        return modoInicializar;
    }

    public void setModoInicializar(boolean modoInicializar) {
        this.modoInicializar = modoInicializar;
    }

    public boolean isMostrarAtaque() {
        return mostrarAtaque;
    }

    public void setMostrarAtaque(boolean mostrarAtaque) {
        this.mostrarAtaque = mostrarAtaque;
    }

    public boolean isMostrarMoverTropas() {
        return mostrarMoverTropas;
    }

    public void setMostrarMoverTropas(boolean mostrarMoverTropas) {
        this.mostrarMoverTropas = mostrarMoverTropas;
    }

    public void setRonda(int r) {
        this.ronda = r;
    }

//metodo que se encarga de pasar de ronda de juego
    public void nuevaRonda() {
        //aumentamos la ronda de juego
        setRonda(getRonda() + 1);

        pasarTurno();

    }

    public int getModoJuego() {
        return modoJuego;
    }

    public void setModoJuego(int modoJuego) {
        this.modoJuego = modoJuego;
    }

    public void prepararModo() {
        Random rnd = new Random();
        //si es modo Capital
        if (modoJuego == 1) {
            //creamos el array de capitales.
            setCapitales(new ArrayList<Territorio>());
            //repartimos para cada jugador una capital
            for (int i = 0; i < jugadores.size(); i++) {
                //cogemos una carta de la baraja, sin coger los 2 comodines
                int cartaAl = rnd.nextInt(cartasTer.size() - 2);
                //mientras el jugador i no tenga su capital
                while (jugadores.get(i).getCapitales() == null) {
                    //si la carta pertenece al jugador
                    if (cartasTer.get(cartaAl).getPais().getPropietarioTer() == jugadores.get(i)) {
                        ArrayList<Territorio> capitalJug = new ArrayList<Territorio>();
                        //le asignamos dicha capital
                        jugadores.get(i).setCapitales(capitalJug);
                        jugadores.get(i).anadirCapital(cartasTer.get(cartaAl).getPais());
                        //añadimos el pais a la lista de capitales
                        getCapitales().add(cartasTer.get(cartaAl).getPais());
                        //marcamos el pais como capital
                        cartasTer.get(cartaAl).getPais().setEsCapital(true);
                        //eliminamos la carta de la baraja
                        cartasTer.remove(cartaAl);
                    } else {
                        //cogemos la carta siguiente
                        cartaAl++;
                        //si hemos llegado al final de la bajara, empezamos de nuevo
                        if (cartaAl >= cartasTer.size() - 2) {
                            cartaAl = 0;
                        }

                    }
                }
            }
        } //si es modo Mision Secreta
        else if (modoJuego == 2) {
        }
    }

    public boolean comprobarFin() {
        int numJug = 0;
        int jugSolitario = 0;
        //primero comprobamos si algun jugador ha sido eliminado
        for (int i = 0; i < jugadores.size(); i++) {
            //si el jugador no tiene territorios
            if (jugadores.get(i).getTerritorio().size() == 0) {
                //lo eliminamos de la lista de jugadores
                jugadores.remove(i);
            }
            //ademas contamos el numero de jugadores, sin contar los neutrales
            if (!jugadores.get(i).isNeutral()) {
                numJug++;
                jugSolitario = i;
            }
        }
        //si solo hay 1 jugador que no es neutral es el ganador
        //y será el que indique jugSolitario
        if (numJug == 1) {
            System.out.println("Tenemos ganador!!");
            ganador = jugadores.get(jugSolitario);
            return true;
        }
        //ahora comprobamos si se han cumplido el numero de rondas maximas
        if ((rondaMax != 0) && (ronda > rondaMax)) {
            System.out.println("Comprobando ganador!");
            //si es modo 0,hacemos que gane el jugador con mas territorios
            int jug = 0;
            int mayor = 0;
            boolean empate = true;
            if (modoJuego == 0) {
                //para cada jugador
                for (int i = 0; i < jugadores.size(); i++) {
                    //comprobamos si su numero de terr es mayor que el del jug anterior
                    //sin contar los jugadores neutrales
                    if ((jugadores.get(i).getTerritorio().size() > mayor) && (!jugadores.get(i).isNeutral())) {
                        mayor = jugadores.get(i).getTerritorio().size();
                        jug = i;
                        empate = false;
                    //si son iguales, marcamos un empate por ahora.
                    //sin contar los jugadores neutrales
                    } else if ((jugadores.get(i).getTerritorio().size() == mayor) && (!jugadores.get(i).isNeutral())) {
                        empate = true;
                    }
                }
                //marcamos como ganador al que tenga mas territorios si no hay empate
                if (!empate) {
                    System.out.println("Tenemos ganador!!");
                    setGanador(jugadores.get(jug));
                    return true;
                } else {
                    Jugador jugempate = new Jugador("¡EMPATE!", 9);
                    setGanador(jugempate);
                    return true;
                }
            } //si es modo 1, gana el jugador con mas capitales, y en caso de empate, el de mas terr
            else if (modoJuego == 1) {
                //para cada jugador
                for (int i = 0; i < jugadores.size(); i++) {
                    //comprobamos si su numero de capitales es mayor que el del jug anterior
                    if ((jugadores.get(i).getCapitales().size() > mayor) && (!jugadores.get(i).isNeutral())) {
                        mayor = jugadores.get(i).getCapitales().size();
                        jug = i;
                        empate = false;
                    } else if ((jugadores.get(i).getCapitales().size() == mayor) && (!jugadores.get(i).isNeutral())) {
                        empate = true;
                    }
                }
                //marcamos como ganador al que tenga mas capitales si no hay empate
                if (!empate) {
                    setGanador(jugadores.get(jug));
                    return true;
                //si hay empate comprobamos los territorios
                } else {
                    jug = 0;
                    mayor = 0;
                    //para cada jugador
                    for (int i = 0; i < jugadores.size(); i++) {
                        //comprobamos si su numero de terr es mayor que el del jug anterior
                        if ((jugadores.get(i).getTerritorio().size() > mayor) && (!jugadores.get(i).isNeutral())) {
                            mayor = jugadores.get(i).getTerritorio().size();
                            jug = i;
                            empate = false;
                        //si son iguales, marcamos un empate por ahora.
                        } else if ((jugadores.get(i).getTerritorio().size() == mayor) && (!jugadores.get(i).isNeutral())) {
                            empate = true;
                        }
                    }
                    //marcamos como ganador al que tenga mas territorios si no hay empate
                    if (!empate) {
                        setGanador(jugadores.get(jug));
                        return true;
                    } else {
                        Jugador jugempate = new Jugador("¡EMPATE!", 9);
                        setGanador(jugempate);
                        return true;
                    }
                }

            } //modo misiones por hacer!
            else {
            }
        } else {
            //si es el modo Dominacion Total
            if (modoJuego == 0) {
                //comprobamos el numero de paises de cada jugador
                for (int i = 0; i < jugadores.size(); i++) {
                    //si ha conquistado todos los paises
                    if (jugadores.get(i).getTerritorio().size() == tablero.getTerritorio().size()) {
                        setGanador(jugadores.get(i));
                        return true;
                    }

                }
            } //si es el modo Capital
            else if (modoJuego == 1) {
                //comprobamos si algun jugador tiene todas las capitales 
                for (int i = 0; i < jugadores.size(); i++) {
                    //variable que cuenta el numero de capitales que tiene un jugador
                    int numCapitales = 0;
                    for (int j = 0; j < getCapitales().size(); j++) {
                        //si la capital pertenece al jugador que analizamos
                        if (getCapitales().get(j).getPropietarioTer() == jugadores.get(i)) {
                            numCapitales++;
                        }

                    }
                    //si un jugador tiene todas las capitales, es el ganador
                    if (numCapitales == getCapitales().size()) {
                        setGanador(jugadores.get(i));
                        return true;
                    }

                }
            } //si es el modo Misiones
            else {
            }
        }

        return false;
    }

    public int getRondaMax() {
        return rondaMax;
    }

    public void setRondaMax(int rondaMax) {
        this.rondaMax = rondaMax;
    }

    public boolean isModoInvasion() {
        return modoInvasion;
    }

    public void setModoInvasion(boolean modoInvasion) {
        this.modoInvasion = modoInvasion;
    }

    public boolean isFinJuego() {
        return finJuego;
    }

    public void setFinJuego(boolean finJuego) {
        this.finJuego = finJuego;
    }

    public Jugador getGanador() {
        return ganador;
    }

    public void setGanador(Jugador ganador) {
        this.ganador = ganador;
    }

    public boolean isCambiarCartas() {
        return cambiarCartas;
    }

    public void setCambiarCartas(boolean cambiarCartas) {
        this.cambiarCartas = cambiarCartas;
    }

    public boolean isCapitalesVisibles() {
        return capitalesVisibles;
    }

    public void setCapitalesVisibles(boolean capitalesVisibles) {
        this.capitalesVisibles = capitalesVisibles;
    }

    public ArrayList<Territorio> getCapitales() {
        return capitales;
    }

    public void setCapitales(ArrayList<Territorio> capitales) {
        this.capitales = capitales;
    }
}

